import { useCallback, useEffect, useState, useMemo } from "react";
import {
  Button,
  TextField,
  Typography,
  CircularProgress,
  Autocomplete,
  Alert,
  Stack
} from "@mui/material";
import { Dialog, GitRefAutocomplete } from "@ess-ics/ce-ui-common";
import {
  GitReference,
  HostInfoWithId,
  useLazyListHostsQuery,
  useLazyListTagsAndCommitIdsQuery
} from "../../../store/deployApi";
import { getErrorMessage } from "../../common/Helper";
import { FormElements } from "../../../types/common";

interface DeployIocFormDefaults {
  name: string;
  description: string;
  git: string;
  gitProjectId: number;
  reference?: string;
  short_reference?: string;
  netBoxHost?: {
    fqdn: string;
    hostId: string;
  };
}

interface IOCDeployDialogProps {
  open: boolean;
  setOpen: (open: boolean) => void;
  iocId: string;
  submitCallback: (arg0: object, arg1: object) => void;
  hasActiveDeployment: boolean;
  deployIocFormDefaults: DeployIocFormDefaults;
  error: string;
  resetError: () => void;
  buttonDisabled: boolean;
  setButtonDisabled: (buttonDisabled: boolean) => void;
}

type SearchMethodConstants = "EQUALS" | "CONTAINS";

export function IOCDeployDialog({
  open,
  setOpen,
  iocId,
  submitCallback,
  hasActiveDeployment,
  deployIocFormDefaults,
  error,
  resetError,
  buttonDisabled,
  setButtonDisabled
}: IOCDeployDialogProps) {
  const [getHosts, { data: hosts, isFetching: loadingHosts }] =
    useLazyListHostsQuery();
  const [
    callgetTagOrCommitIds,
    {
      data: tagOrCommitIds = [],
      isFetching: loadingTagsAndCommitIds,
      error: tagOrCommitIdsError
    }
  ] = useLazyListTagsAndCommitIdsQuery();

  const getTagOrCommitIds = useCallback(
    (
      gitProjectId: number,
      reference: string,
      searchMethod: SearchMethodConstants
    ) => {
      callgetTagOrCommitIds({
        projectId: gitProjectId,
        reference,
        searchMethod
      });
    },
    [callgetTagOrCommitIds]
  );

  const initHost = useMemo(
    () => ({
      fqdn: deployIocFormDefaults.netBoxHost?.fqdn,
      hostId: deployIocFormDefaults.netBoxHost?.hostId
    }),
    [deployIocFormDefaults.netBoxHost]
  );

  const [host, setHost] = useState(initHost);
  const [query, setQuery] = useState("");
  const [gitRepo, setGitRepo] = useState(deployIocFormDefaults.git);
  const [revision, setRevision] = useState(deployIocFormDefaults.reference);
  const gitProjectId = deployIocFormDefaults.gitProjectId;
  const hasHostData = host?.hostId || host?.fqdn;

  const handleClose = () => {
    setOpen(false);
    setHost(initHost);
    resetError();
  };

  useEffect(() => {
    if (!hasHostData && query && query.length > 2) {
      getHosts({ text: `${query}` });
    }
  }, [query, getHosts]);

  useEffect(() => {
    getTagOrCommitIds(gitProjectId, "", "CONTAINS");
  }, [gitProjectId, getTagOrCommitIds]);

  const onSubmit = (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    setButtonDisabled(true);
    const { git: gitText } = event.currentTarget
      .elements as FormElements<"git">;
    const git = gitText.value;

    submitCallback(
      {
        ioc_id: iocId
      },
      {
        requestBody: {
          sourceUrl: git,
          sourceVersion: revision,
          hostId: host.hostId,
          type: "DEPLOY"
        }
      }
    );
  };

  return (
    <Dialog
      open={open}
      onClose={handleClose}
      title={
        <Typography
          variant="h2"
          marginY={1}
        >
          {hasActiveDeployment ? "Deploy revision" : "Deploy"}
        </Typography>
      }
      content={
        <Stack
          component="form"
          onSubmit={onSubmit}
          gap={1}
        >
          <TextField
            autoComplete="off"
            id="git"
            label="Git repository"
            variant="standard"
            defaultValue={deployIocFormDefaults.git || ""}
            fullWidth
            onChange={(event) => {
              setGitRepo(event.target.value);
              resetError();
            }}
            disabled
            required
          />
          <GitRefAutocomplete
            id="version"
            label="Revision"
            options={tagOrCommitIds}
            loading={loadingTagsAndCommitIds}
            disabled={!gitRepo || gitRepo.trim() === ""}
            autocompleteProps={{
              autoSelect: false,
              defaultValue: tagOrCommitIds.find(
                (tagOrCommit) =>
                  tagOrCommit.reference === deployIocFormDefaults.reference
              )
            }}
            onGitQueryValueSelect={(
              _: React.SyntheticEvent<Element, Event>,
              value: GitReference
            ) => {
              setRevision(value?.reference);
              resetError();
            }}
            textFieldProps={{
              helperText:
                tagOrCommitIdsError && getErrorMessage(tagOrCommitIdsError)
            }}
          />

          <Autocomplete
            autoHighlight
            id="host"
            options={query ? hosts?.netBoxHosts ?? [] : []}
            loading={loadingHosts}
            clearOnBlur={false}
            value={host}
            getOptionLabel={(option) => {
              return option?.fqdn ?? "";
            }}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Host"
                variant="outlined"
                required
                InputProps={{
                  ...params.InputProps,
                  endAdornment: (
                    <>
                      {loadingHosts ? (
                        <CircularProgress
                          color="inherit"
                          size={20}
                        />
                      ) : null}
                      {params.InputProps.endAdornment}
                    </>
                  )
                }}
              />
            )}
            onChange={(_, value: HostInfoWithId | null) => {
              setHost({ fqdn: value?.fqdn, hostId: value?.hostId });
              resetError();
            }}
            onInputChange={(_, value) => setQuery(value)}
            disabled={hasActiveDeployment}
            autoSelect
            filterOptions={(options) => options}
          />

          {hasActiveDeployment ? (
            <Typography variant="body2">
              Hint: First undeploy IOC if you want to move to another host.
            </Typography>
          ) : (
            <></>
          )}

          {error ? <Alert severity="error">{error}</Alert> : <></>}
          <Stack
            flexDirection="row"
            justifyContent="flex-end"
            gap={2}
            marginTop={1}
          >
            <Button
              onClick={handleClose}
              color="primary"
            >
              Cancel
            </Button>
            <Button
              color="primary"
              variant="contained"
              type="submit"
              disabled={!host || !revision || buttonDisabled}
            >
              Deploy
            </Button>
          </Stack>
        </Stack>
      }
    />
  );
}
